import re
from openai import AzureOpenAI
import time
import constants
from LlmCallLogger import LLMCallLogger


# Set up OpenAI API configuration

llm_log_file_name = "llm_call_log.txt"

def RemoveCodeMarkersFromLLMResponse(response):
    # Remove lines starting with \n``` or ```
    cleaned_response = re.sub(r'\n```.*|```.*', '', response)
    return cleaned_response

def CallAzureOpenAI4o(prompt, temperature = 0.7, silent = False, top_p = .95):

    # truncate the prompt if it is too long
    token_limit = 80000 # 1 token = 4 characters
    if (len(prompt) > token_limit * 4):        
        prompt = prompt[:token_limit * 4]
        print(f"Truncated prompt to length: {len(prompt)}")

    messages = [{"role": "user", "content": prompt}]        
    
    if not silent:
        print(f"Starting Azure OpenAI Call, prompt length (chars): {len(str(messages))}")
    
    start = time.time()

    # Setting up the deployment name
    client = AzureOpenAI(
        api_key=constants.api_key4o,  
        api_version=constants.api_version4o,
        azure_endpoint = constants.azure_endpoint4o,
        timeout = 10000,
        max_retries = 3,
    )
                

    completion = client.chat.completions.create(
        model=constants.model4o, 
        temperature=temperature,
        messages=messages,
        max_tokens=4096,    
        top_p=top_p,
        frequency_penalty=0,
        presence_penalty=0,
    )

    response = str(completion.choices[0].message.content)

    model = completion.model
    completion_tokens = completion.usage.completion_tokens
    prompt_tokens = completion.usage.prompt_tokens

    logger = LLMCallLogger(llm_log_file_name)    
    logger.log(model, completion_tokens, prompt_tokens)


    response = response.replace("```json","").replace("\n```","")

    end = time.time()

    if not silent:
        print(f"Finished Azure OpenAI Call, duration: {str(end - start)}, response length (chars):{len(str(response))}")
    return response

def CallAzureOpenAI4oMini(prompt, temperature = 0.7, silent = False, top_p = .95):

    # truncate the prompt if it is too long
    token_limit = 80000 # 1 token = 4 characters
    if (len(prompt) > token_limit * 4):        
        prompt = prompt[:token_limit * 4]
        print(f"Truncated prompt to length: {len(prompt)}")

    messages = [{"role": "user", "content": prompt}]        
    
    if not silent:
        print(f"Starting Azure OpenAI Call, prompt length (chars): {len(str(messages))}")
    
    start = time.time()

    # Setting up the deployment name
    client = AzureOpenAI(
        api_key=constants.api_key4o_mini,  
        api_version=constants.api_version4o_mini,
        azure_endpoint = constants.azure_endpoint4o_mini,
        timeout = 10000,
        max_retries = 3,
    )
                
    completion = client.chat.completions.create(
        model=constants.model40_mini, 
        temperature=temperature,
        messages=messages,
        max_tokens=4096,    
        top_p=top_p,
        frequency_penalty=0,
        presence_penalty=0,
    )

    response = str(completion.choices[0].message.content)

    model = completion.model
    completion_tokens = completion.usage.completion_tokens
    prompt_tokens = completion.usage.prompt_tokens

    logger = LLMCallLogger(llm_log_file_name)    
    logger.log(model, completion_tokens, prompt_tokens)


    response = response.replace("```json","").replace("\n```","")

    end = time.time()

    if not silent:
        print(f"Finished Azure OpenAI Call, duration: {str(end - start)}, response length (chars):{len(str(response))}")
    return response

