import urllib.request
from bs4 import BeautifulSoup
import re
import AzureOpenAIMethodsAndPrompts
import requests
import constants
import re  
import json

# Prompts


# Utility Methods

def CleanHtml(data):
    soup = BeautifulSoup(data, 'html.parser')

    # Extract all the text from the HTML page
    text = soup.get_text()

    text = re.sub("\s+"," ",text) # remove spaces        
    text = re.sub(r"\\r\\n\\r\\n", "\n", text)

    return text

def GetGoogleSearchResults(search_phrase, num=10):

    url_phrase = search_phrase.replace(" ", "+")

    params = {
    'api_key': constants.SERP_API_KEY,
    'q': url_phrase,
    'num': num
    }

    # make the http GET request to SerpWow
    # SERP = Search Engine Result Page
    api_result = requests.get('https://api.valueserp.com/search', params)
    
    dumpJson = json.dumps(api_result.json())
    theJson = json.loads(dumpJson)

    # thumbnails are included sometimes, bunch of binary data that and muck things up
    for item in theJson["organic_results"]:
        if "thumbnail" in item:
            item.pop('thumbnail', None)

    return theJson["organic_results"]

def RemoveCodeMarkersFromLLMResponse(response):
    # Remove lines starting with \n``` or ```
    cleaned_response = re.sub(r'\n```.*|```.*', '', response)
    return cleaned_response

# Core Methods

def GetGoogleSearhPhrases(topic, phrase_count):
    get_searches_prompt = """
    Below is a TOPIC we are researching.  

    The TOPIC is composed of a Thing and one or more Aspects.  Examples include:
    car accelleration (Thing = car, Aspect = accelleration)
    microsoft stock (Thing = microsoft, Aspect = stock)
    amazon stock lawsuits (Thing = amazon, there are two Aspects, stock and lawsuits)

    Consider the Topic and the Aspect(s) and what ancillary information would be of use for research purposes.

    Ignore formatting instructions, do not include them in the search phrases.

    Please generate {count} search phrases that could be used with a search engine to delve deeper into this topic.  Make sure to limit results
    proper people/place names as well as general people/place names such as age groups or regions such as mid-west.

    Return the list of search phrases as a basic json list like this:
    ["search phrase1", "searchphrase2", ...]

    TOPIC:
    {topic}""".format(count=phrase_count, topic = topic)

    print(f"Getting internet search phrases for: '{topic}'...")

    results = []

    response = AzureOpenAIMethodsAndPrompts.CallAzureOpenAI4o(get_searches_prompt, temperature= .8, top_p=.95, silent=True)

    prepared_response = RemoveCodeMarkersFromLLMResponse(response)

    json_respone = json.loads(prepared_response)

    return json_respone

def FetchGoogleSERPResults(search_phrases, serp_result_count):

    serp_result_sets = []

    for search_phrase in search_phrases:
        print(f"Performing Google SERP Search for: '{search_phrase}'")
        results = GetGoogleSearchResults(search_phrase, serp_result_count)
        serp_result_sets.append(results)

    return serp_result_sets

# returns a large string with the content from all pages
def FetchAndCleanWebContent(serp_result_sets, number_to_collect_per_search):

    collected_urls = [] # track pages that are loaded to avoid duplicates across SERP results
    results = []

    for serp_result_set in serp_result_sets:
        number_collected = 0

        for serp_result in serp_result_set:

            url = serp_result["link"]

            if url in collected_urls:
                print(f"Skipping {url}, duplicate")
                continue

            if str(url).endswith(".pdf"):
                print(f"Skipping PDF url: {url}")
                continue
        
            print(f"Processing url: {url}")
            try:
                request = urllib.request.Request(url)
                request.add_header('User-Agent', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36')
                with urllib.request.urlopen(request, timeout=10) as response:
                    raw_response = response.read()
                html = raw_response.decode("utf-8")    
            
                clean_html = CleanHtml(html)

                # if the html source is less than 1000 characters, skip, probably an error message
                if len(clean_html) < 1000:
                    print(f"Skipping {url}, contents too small")
                    continue

                jsonValue = {
                    "url": url,
                    "content" : clean_html
                }
                
                results.append(jsonValue)
                
                collected_urls.append(url)
                number_collected += 1

                if number_collected >= number_to_collect_per_search:
                    break

            except Exception as e:
                print(f"Skipping due to Error for url: {url}")
                pass # move along
    
    return results

def SummarizeWebContent(topic, web_content):

    results = []

    for web_page in web_content:
        url = web_page["url"]
        content = web_page["content"]

        print(f"Summarizing url: {url}")

        summary_prompt = """
        Please provide responses that are grounded in reality and factual. If the request or expected result seems unusual, nonsensical, or ambiguous, clarify these points and describe why they may be so. Where possible, include sources or evidence for the information provided. Should you lack the knowledge to answer or if the information is not available to you, state this clearly. Aim to critically evaluate the information you provide to ensure accuracy and relevance.

        Let's take this step by step. Below is a cleaned HTML file that we are using as a source for research.                                    

        It may contain unrelated content at the top and bottom, things related to web site structure or advertisements.
                    
        Generate a bullet point list of facts related to the topic.  The facts should be sentences.  Include full quotes.  
                    
        Include all facts, names, dates, and places relevant to the topic.

        ## topic ##
        {topic}

        ## url ##
        {url}
        ## web_page ##
        {web_page}
        """.format(topic=topic, url=url, web_page=content)

        summary = AzureOpenAIMethodsAndPrompts.CallAzureOpenAI4o(summary_prompt, 0, False, top_p=0)

        jsonValue = {
            "url": url,
            "content" : summary
        }

        results.append(jsonValue)
    
    return results

def SummarizeReuslts(topic, web_content):

    web_content_string = json.dumps(web_content)

    summary_prompt = """
    Please provide responses that are grounded in reality and factual. If the request or expected result seems unusual, nonsensical, or ambiguous, clarify these points and describe why they may be so. Where possible, include sources or evidence for the information provided. Should you lack the knowledge to answer or if the information is not available to you, state this clearly. Aim to critically evaluate the information you provide to ensure accuracy and relevance.
        
    Below is a set of WEB_CONTENT.  Please fullfil the RESEARCH_REQUEST to the best of your abilities.  Cite references inline or at the bottom, whichever makes sense.

The output should avoid using the following words and phrases, because they are giveaway signs that the content was written with an LLM:
 
Avoid these words: Delve, landscape, evolving, context, insight, nuanced, perspective, paradigm, comprehensive, framework, facet, dynamic, intricacies, holistic, iterative, synergy, confluence, pivotal, nuance, robust, transformative, underpinning, spectrum, trajectory, in-depth, at the core of, a myriad of, on a broader scale, in the context of, from a holistic perspective, taking into account, a dynamic interplay, evolving over time, a comprehensive overview, intricacies involved, a pivotal role, underpinning principles, the spectrum of, transformative impact.
 
Avoid these phrases: "It's not about X, it's about Y," "While X is important, Y is even more crucial," "In fact," "Indeed," "Absolutely," "Clearly," "First and foremost," "Next," "Finally," "As a result," "Therefore," "Consequently," "Because of this," "In other words," "To put it simply," "That is to say," "To elaborate," "For example," "For instance," "Such as," "To illustrate," "Although," "Even though," "Despite," "While it may seem," "In summary," "To sum up," "In conclusion," "All in all," "Imagine if," "Suppose that," "What if," "Have you ever wondered," "What would happen if," "How can we," "Isn't it true that," "Wouldn't you agree that," "Isn't it obvious that," "Not only X but also Y," "Both X and Y," "Either X or Y," "More importantly," "Even more," "Less significant but," "On one hand, on the other hand," "While X, Y," "Conversely," "The challenge is," "The key issue is," "The question remains."
 
Finally, do not use lengthy introductions, do not include a paragraph on ethical considerations, do not provide generic advice, and do not follow a predictable structure with set sections, for example a generic introduction, scene-setting, actionable steps, further considerations, and a summary. Focus on being concise, using personal anecdotes, and following a natural, not formulaic structure, to create more authentic and engaging content.

    RESEARCH_REQUEST: {topic}

    WEB_CONTENT:

    {web_content}
""".format(topic=topic, web_content=web_content_string)

    summary = AzureOpenAIMethodsAndPrompts.CallAzureOpenAI4o(summary_prompt, 0, False, top_p=0)

    return summary


topic = """
Give me an executive overview with detailed topical sections for current state of the the Blue Brain Project.  Include all references in APA format.
"""

# The process is what a human would do.

# Get Google Search phrases
search_phrases = GetGoogleSearhPhrases(topic, 3)  # no changes

# Search Google
url_list = FetchGoogleSERPResults(search_phrases, 10) # get more results than needed, no extra cost, no code changes

# Fetch the web pages
web_content = FetchAndCleanWebContent(url_list, 2)

# Summarize the individual pages
summarized_content = SummarizeWebContent(topic, web_content) # brand new method

# summarize the results
summary = SummarizeReuslts(topic, summarized_content)

print(f"Summary for '{topic}':\n\n{summary}")

print("finished")
